// Autor: Frane Kurtovic

#include <cstdio>
#include <cstring>
#include <cassert>
#include <algorithm>
#include <iostream>
#include <string>

using namespace std;


typedef pair <int, int> par;

const char *tipkovnica[3] = {"QWERTYUIOP", "ASDFGHJKL", "ZXCVBNM"};
const int H = 200, W = 300;

par nadji(char tipka) {
  for(int i = 0; i < 3; i++) {
    for(int j = 0; j < tipkovnica[i][j]; j++) {
      if (tipkovnica[i][j] == tipka) return par(i, j);
    }
  }
  return par(-1, -1); // greska
}

par srediste(char tipka) {
  const int RAZMAK = 20;
  const par Q(0, 440+H/2);
  const int X_POC[3] = {0, 150, 470};

  par tipka_pos = nadji(tipka);
  par sol;
  sol.second = Q.second - tipka_pos.first * (H + RAZMAK);
  sol.first = X_POC[tipka_pos.first] + tipka_pos.second * (W + RAZMAK) + W/2;
  return sol;
}

inline int dist(const par& a, const par& b) {
  return (a.first-b.first)*(a.first-b.first) +
    (a.second-b.second)*(a.second-b.second);
}

const int INF = 1 << 29;
const int MXN = 10+1;
par tocke[MXN];
par mid[200];

bool ok(char slovo, par tocka) {
  par s = mid[slovo];
  return abs(s.first-tocka.first) <= W/2 && abs(s.second-tocka.second) <= H/2;
}

int n, m, a, d;

int dist(char tipka, const par& tocka) {
  par tipka_pos = mid[tipka];
  return dist(tipka_pos, tocka);
}

int run_cost(const string& cilj) {
  int ret = 0;
  for (int i = 0; i < (int)cilj.size(); i++) {
    ret += dist(cilj[i], tocke[i]);
  }
  return ret;
}

int main() {
  for (char slovo = 'A'; slovo <= 'Z'; slovo++) {
    mid[slovo] = srediste(slovo);
  }

  cin >> n >> m;
  for (int i = 0; i < n; i++) {
    cin >> tocke[i].first >> tocke[i].second;
    char best_slovo = '?';
    for (char slovo = 'A'; slovo <= 'Z'; slovo++) {
      if (ok(slovo, tocke[i])) {
        best_slovo = slovo;
        break;
      }
    }
    cout << best_slovo;
  }
  cout << endl;
  string best_word = "?";
  int best_cost = INF;
  for (int i = 0; i < m; i++) {
    string rijec;
    cin >> rijec;
    int curr_cost = run_cost(rijec);
    if (curr_cost < best_cost) {
      best_cost = curr_cost;
      best_word = rijec;
    }
  }
  assert(best_word != "?");
  cout << best_word << endl;
  return 0;
}
